(*******************************
*  TmsScrollImage version 1.1  *
*  (c) 2007-2009 MarcaSoft     *
*******************************)
unit msScrollImage;
 
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;


type
  TmsScrollImage = class(TCustomPanel)
  private
    { Private declarations }
    FirstRun: boolean;
    xX, xY: integer;
    fOnChange: TNotifyEvent;
    fOverSize: boolean;
    fScaled: boolean;
    fGrabMode: boolean;
    fScrollPicture: TPicture;
    fHintPic: string;
    FHintScr: string;
    fTrackView: boolean;
    FOnProgress: TProgressEvent;
    iPic: TImage;
    pHScroll, pVScroll: TScrollBar;
    fBevel: TPanelBevel;
    procedure DoDesigning;
    procedure iPicOnChange(Sender: TObject);
    procedure SetBevel(aBevel: TPanelBevel);
    procedure SetOverSize(aOverSize: boolean);
    procedure SetGrabMode(aGrabMode: boolean);
    procedure SetScaled(aScaled: boolean);
    procedure SetHintPic(aHint: string);
    procedure SetHintScr(aHint: string);
    procedure SetScrollPicture(aScrollPicture: TPicture);
    procedure ShowPicSizes;
    procedure pHScrollChange(Sender: TObject);
    procedure pVScrollChange(Sender: TObject);
    procedure pHScrollScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: integer);
    procedure pVScrollScroll(Sender: TObject; ScrollCode: TScrollCode;
       var ScrollPos: integer);
    procedure iPicMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: integer);
    procedure iPicMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: integer);
    procedure iPicMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: integer);
    procedure ScrollResize(Sender: TObject);
  protected
    { Protected declarations }
    procedure Paint; override;
    property FullRepaint;
    property ParentBackGround;
  public
    { Public declarations }
    constructor
      Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure RefreshPicture;
    function IsOverPicture: boolean;
    procedure RemovePicture;
  published
    { Published declarations }
    property AllowOverSize: boolean
      read fOverSize write SetOverSize default false;
    property Scaled: boolean
      read fScaled write SetScaled default false;
    property HintPicture: string
      read fHintPic write SetHintPic;
    property HintScrollBar: string
      read fHintScr write SetHintScr;
    property Picture: TPicture
      read fScrollPicture write SetScrollPicture;
    property OnProgress: TProgressEvent
      read FOnProgress write FOnProgress;
    property GrabMode: boolean
      read fGrabMode write SetGrabMode default true;
    property TrackView: boolean
      read fTrackView write fTrackView default False;
    property Bevel: TPanelBevel
      read fBevel Write SetBevel default bvLowered;
    property OnChange: TNotifyEvent
      read FOnChange write FOnChange;
    property Caption;
    property Font;
    property ParentFont;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property Align;
    property Color;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Height;
    property Left;
    property Name;
    property ParentColor;
    property Tag;
    property Top;
    property Visible;
    property Width;
    property ShowHint;
    property ParentShowHint;
end;


procedure Register;

implementation

const
  crCustomcrGrab      = 15;
  crCustomcrMove      = 16;

{$R ScrollImageCursors.RES}

procedure Register;
begin
  RegisterComponents('MarcaSoft', [TmsScrollImage]);
end;

destructor TmsScrollImage.destroy;
begin
  inherited Destroy;
end;

procedure TmsScrollImage.RemovePicture;
begin
  ipic.free;
  iPic:=TImage.create(self);
  iPic.parent:=self;
  iPic.width:=0;
  iPic.height:=0;
  iPic.left:=1;
  iPic.top:=1;
  iPic.OnMousedown:=iPicMousedown;
  iPic.OnMouseup:=iPicMouseup;
  iPic.OnMousemove:=iPicMousemove;
  iPic.onClick:=Onclick;
  iPic.OnDblClick:=OnDblClick;
  iPic.OnDragDrop:=OnDragDrop;
  iPic.OnDragOver:=onDragOver;
  iPic.OnEndDrag:=onEndDrag;
  iPic.OnStartDrag:=OnStartDrag;
  iPic.OnProgress:=OnProgress;
  iPic.Picture.OnChange:=iPicOnChange;
  iPic.controlstyle:=iPic.controlstyle+[csOpaque];
  fscrollPicture:=iPic.Picture;
  if ShowHint=true then begin
    iPic.ShowHint:=true;
  end else begin
    iPic.ShowHint:=false;
  end;
  RefreshPicture;
end;

constructor
  TmsScrollImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  parentcolor:=true;
  ParentBackGround:=false;
  controlstyle:=controlstyle-[csAcceptsControls];
  controlstyle:=controlstyle+[csOpaque];
  FirstRun:=true;
  xX:=-1;
  xY:=-1;
  width:=105;
  height:=105;
  iPic:=TImage.create(self);
  iPic.parent:=self;
  if (csDesigning in ComponentState)
  and (iPic.Picture.width=0) then begin
    iPic.width:=103;
    iPic.height:=103;
  end else if (iPic.Picture.width=0) then begin
    iPic.width:=0;
    iPic.height:=0;
  end;
  iPic.left:=1;
  iPic.top:=1;
  iPic.OnMousedown:=iPicMousedown;
  iPic.OnMouseup:=iPicMouseup;
  iPic.OnMousemove:=iPicMousemove;
  iPic.onClick:=Onclick;
  iPic.OnDblClick:=OnDblClick;
  iPic.OnDragDrop:=OnDragDrop;
  iPic.OnDragOver:=onDragOver;
  iPic.OnEndDrag:=onEndDrag;
  iPic.OnStartDrag:=OnStartDrag;
  iPic.OnProgress:=OnProgress;
  iPic.Picture.OnChange:=iPicOnChange;
  iPic.controlstyle:=iPic.controlstyle+[csOpaque];
  pVScroll:=TScrollBar.create(self);
  pVScroll.parent:=self;
  pVScroll.kind:=sbVertical;
  pVScroll.height:=88;
  pVScroll.left:=88;
  pVScroll.top:=0;
  pVScroll.visible:=false;
  pHScroll:=TScrollBar.create(self);
  pHScroll.parent:=self;
  pHScroll.kind:=sbHorizontal;
  pHScroll.width:=100;
  pHScroll.left:=0;
  pHScroll.top:=88;
  pHScroll.visible:=false;
  pHScroll.OnChange:=pHScrollChange;
  pVScroll.OnChange:=pVScrollChange;
  onresize:=Scrollresize;
  pVScroll.onScroll:=pVScrollScroll;
  pHScroll.onScroll:=pHScrollScroll;
  fGrabMode:=true;
  fScaled:=false;
  fOverSize:=false;
  fTrackView:=false;
  fBevel:=bvLowered;
  fScrollPicture:=iPic.Picture;
  FullRepaint:=true;
  if ShowHint=true then begin
    iPic.ShowHint:=true;
    pVScroll.ShowHint:=true;
    pHScroll.ShowHint:=true;
  end else begin
    iPic.ShowHint:=false;
    pVScroll.ShowHint:=false;
    pHScroll.ShowHint:=false;
  end;
  Screen.Cursors[crCustomcrGrab] := LoadCursor(HInstance, 'crgrab');
  Screen.Cursors[crCustomcrMove] := LoadCursor(HInstance, 'crmove');
end;

procedure TmsScrollImage.iPicOnChange(Sender: TObject);
begin
  if assigned(OnChange) then
    OnChange(Sender);
  RefreshPicture;
end;

procedure TmsScrollImage.SetBevel(aBevel: TPanelBevel);
begin
  fBevel:=aBevel;
  BevelOuter:=fBevel;
  refresh;
end;

function TmsScrollImage.IsOverPicture: boolean;
var
  txX, tyY, bxX, byY: integer;
  CurPos: TPoint;
  tPosition, bPosition: TPoint;
  vOffSet, hOffSet: integer;
begin
  VOffSet:=10;
  HOffSet:=10;
  IsOverPicture:=true;
  GetCursorPos(CurPos);
  bxX:=0;
  byY:=0;
  txX:=0;
  tyY:=0;
  if not pHScroll.visible then begin
    VOffSet:=0;
    HOffSet:=0;
  end else begin
    if iPic.width<width-2 then
      HOffSet:=0;
    if iPic.height<height-2 then
      VOffSet:=0;
  end;
  if iPic.Picture.width>0 then begin
    if iPic.width>width-2 then
      bxX:=width-2
    else
      bxX:=iPic.width+1;
    if iPic.height>height-2 then
      byY:=height-2
    else
      byY:=iPic.height+1;
  end;
    tPosition:=ClientToScreen(Point(txX,tyY));
    bPosition:=ClientToScreen(Point(bxX,byY));
    if (tPosition.x>=CurPos.x)
    or (bPosition.x-hOffSet<=CurPos.x)
    or (tPosition.y>=CurPos.y)
    or (bPosition.y-vOffSet<=CurPos.y) then
      IsOverPicture:=false;
end;

procedure TmsScrollImage.DoDesigning;
begin
  if csDesigning in ComponentState then begin
    if iPic.Picture.width=0 then begin
      iPic.width:=self.width-2;
      iPic.height:=self.height-2;
      pVScroll.top:=-1000;
      pVScroll.left:=-1000;
      pHScroll.top:=-1000;
      pHScroll.left:=-1000;
      iPic.ControlStyle:=iPic.ControlStyle-[csOpaque];
    end else if (fScaled)
    or ((iPic.width<=self.width-2)
    and (iPic.height<=self.height-2)) then begin
      pVScroll.top:=-1000;
      pVScroll.left:=-1000;
      pHScroll.top:=-1000;
      pHScroll.left:=-1000;
    end else begin
      pVScroll.Left := self.width-(pVScroll.width);
      pHScroll.Top := self.height-(pHScroll.height);
      pHScroll.left:=0;
      pVScroll.top:=0;
      pHScroll.width := pVScroll.Left+pVScroll.width;
      pVScroll.height := pHScroll.Top;
    end;
  end else begin
    iPic.ControlStyle:=iPic.ControlStyle+[csOpaque];
  end;
  refresh;
end;

procedure TmsScrollImage.Paint;
begin
  inherited Paint;
  if FirstRun then begin
    if csDesigning in ComponentState then
      RefreshPicture;
      FirstRun:=false;
  end;
end;

procedure TmsScrollImage.RefreshPicture;
begin
  showpicsizes;
  doDesigning;
end;

procedure TmsScrollImage.SetScrollPicture(aScrollPicture: TPicture);
begin
  fScrollPicture.assign(aScrollPicture);
  if iPic.Picture.width=0 then begin
    if not (csDesigning in ComponentState) then begin
      iPic.width:=0;
      iPic.height:=0;
    end else begin
      iPic.width:=self.width-2;
      iPic.height:=self.width-2;
    end;
  end;
  RefreshPicture;
end;

procedure TmsScrollImage.SetGrabMode(aGrabMode: boolean);
begin
  fGrabMode:=aGrabMode;
  if fGrabMode then begin
    if pVScroll.visible then
      iPic.cursor:=crcustomcrgrab
    else
      iPic.cursor:=crDefault;
  end else
    iPic.cursor:=crDefault;
end;

procedure TmsScrollImage.SetHintPic(aHint: string);
begin
  iPic.Hint:=aHint;
  fHintPic:=aHint;
end;

procedure TmsScrollImage.SetHintScr(aHint: string);
begin
  pVScroll.Hint:=aHint;
  pHScroll.Hint:=aHint;
  fHintScr:=aHint;
end;


procedure TmsScrollImage.SetOverSize(aOverSize: boolean);
begin
  fOverSize:=aOverSize;
  if not allowOverSize then begin
    iPic.width:=iPic.Picture.width;
    iPic.height:=iPic.Picture.height;
  end;
  if Scaled then begin
      RefreshPicture;
  end;
end;

procedure TmsScrollImage.SetScaled(aScaled: boolean);
begin
  fScaled:=aScaled;
  RefreshPicture;
end;

procedure TmsScrollImage.ShowPicSizes;
var
  xM,xW,xH,xT : integer;
  yB : boolean;
  rN : currency;
begin
  if (iPic.Picture.width <> 0) then begin
    rN := iPic.Picture.height/iPic.Picture.width;
    xW := self.width;
    if (not fScaled)
    or ((fScaled)
    and (not fOverSize)
    and ((xW-2) > (iPic.Picture.width))) then
      xW := iPic.Picture.width+2;
    xH := self.height;
    xT := Round(xW * rN);
    if (xT < xH) then
      xH := xT
    else
      xW := Trunc(xH / rN);
    if (not fScaled) and ((iPic.Picture.width > (self.width-2))
    or (iPic.Picture.height > (self.height-2))) then begin
      iPic.width := iPic.Picture.width;
      iPic.height := iPic.Picture.height;
    end else begin
      if (fScaled)
      and (not fOverSize)
      and (iPic.Picture.width<xW) then
        iPic.width:=iPic.Picture.width
      else
        iPic.width := xW-2;
      if (fScaled)
      and (not fOverSize)
      and (iPic.Picture.height<xH) then
        iPic.height:=iPic.Picture.height
      else
        iPic.height := xH-2;
    end;
  end;
  if (fScaled)
  and (not fOverSize)
  and (iPic.width<self.width-2)
  and (iPic.height<self.height-2) then
    iPic.stretch:=false
  else if not fScaled then
    iPic.stretch:=false
  else
    iPic.stretch:=true;
  if fScaled then begin
    pHScroll.Position := 0;
    pVScroll.Position := 0;
    pHScroll.Visible := false;
    pVScroll.Visible := false;
    iPic.Cursor := crDefault;
  end else begin
    if (iPic.Picture.width > self.width-2)
    or (iPic.Picture.height > self.height-2) then begin
      pVScroll.visible:=true;
      pHScroll.visible:=true;
      if grabmode then
        ipic.cursor:=crcustomcrgrab;
    end else begin
      pHScroll.Position := 0;
      pVScroll.Position := 0;
      pHScroll.Visible := false;
      pVScroll.Visible := false;
      iPic.Cursor := crDefault;
      ipic.cursor:=crdefault;
    end;
      if iPic.Picture.width < self.width-pVScroll.width-1 then
        pHScroll.max:=0
      else begin
        xM := iPic.Picture.width-(self.width-2)+pVScroll.width;
        if (xM > 0) then begin
          yB := (xM > pHScroll.Max);
          if pHScroll.max = 0 then
            rn := 0
          else
            rN := (xM / pHScroll.Max);
          if (yB) then begin
            pHScroll.Max := xM;
            if fTrackView then
              pHScroll.Position := Round(pHScroll.Position*rN);
          end else begin
            if fTrackView then
              pHScroll.Position := Round(pHScroll.Position*rN);
            pHScroll.Max := xM;
          end;
        end;
      end;
      if iPic.Picture.height < self.height-pHScroll.height-1 then
        pVScroll.max:=0
      else begin
        xM := iPic.Picture.height-(self.height-2)+pHScroll.height;
        if (xM > 0) then begin
          yB := (xM > pVScroll.Max);
          if pVScroll.max = 0 then
            rn := 0
          else
            rN := (xM / pVScroll.Max);
          if (yB) then begin
            pVScroll.Max := xM;
            if fTrackView then
              pVScroll.Position := Round(pHScroll.Position*rN);
          end else begin
            if fTrackView then
              pVScroll.Position := Round(pHScroll.Position*rN);
            pVScroll.Max := xM;
          end;
        end;
      end;
  end;
  ipic.refresh;
end;

procedure TmsScrollImage.pHScrollChange(Sender: TObject);
begin
  iPic.Left := 1-pHScroll.Position;
  PHScroll.Refresh;
  PVScroll.Refresh;
end;

procedure TmsScrollImage.pVScrollChange(Sender: TObject);
begin
  iPic.Top := 1-pVScroll.Position;
  pVScroll.Refresh;
  PHScroll.Refresh;
end;

procedure TmsScrollImage.pHScrollScroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: integer);
begin
  if (ScrollCode = scPageUp) then
    ScrollPos := pHScroll.Position-64
  else if (ScrollCode = scPageDown) then
    ScrollPos := pHScroll.Position+64;
end;

procedure TmsScrollImage.pVScrollScroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: integer);
begin
  if (ScrollCode = scPageUp) then
    ScrollPos := pVScroll.Position-48
  else if (ScrollCode = scPageDown) then
    ScrollPos := pVScroll.Position+48;
end;

procedure TmsScrollImage.iPicMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: integer);
begin
  if assigned(OnMouseDown) then
    OnMouseDown(Sender,Button,shift,x,y);
  if (Button = mbLeft) then begin
    if GrabMode then begin
      xX := X;
      xY := Y;
    end;
    if (not fScaled)
    and ((iPic.Picture.width > (self.width-2))
    or (iPic.Picture.height > (self.height-2))) then
      if GrabMode then
        Screen.Cursor := crcustomcrmove
      else
        Screen.cursor:=crDefault;
  end;
end;

procedure TmsScrollImage.iPicMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: integer);
begin
  if assigned(OnMouseUp) then
    OnMouseUp(Sender,Button,Shift,x,y);
  xX := -1;
  xY := -1;
  Screen.Cursor := crDefault;
end;

procedure TmsScrollImage.iPicMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: integer);
var
  xXD, xYD : integer;
begin
  if assigned(OnMouseMove) then
    OnMouseMove(self,Shift,x,y);
  if (not fScaled)
  and ((iPic.Picture.width > (self.width-2))
  or (iPic.Picture.height > (self.height-2)))
  and (xX >= 0) and (xY >= 0) then begin
    if GrabMode then begin
      xXD := xX-X;
      xYD := xY-Y;
      pHScroll.Position := 0-(iPic.Left-xXD);
      pVScroll.Position := 0-(iPic.Top-xYD);
    end;
  end;
end;

procedure TmsScrollImage.ScrollResize(Sender: TObject);
begin
  pVScroll.Left := self.width-(pVScroll.width);
  pHScroll.Top := self.height-(pHScroll.height);
  pHScroll.width := pVScroll.Left+pVScroll.width;
  pVScroll.height := pHScroll.Top;
  RefreshPicture;
end;

end.
